import numpy as np
from scipy.linalg import lu

def lu_decomposition(matrix, results):
    """
    Resuelve un sistema de ecuaciones lineales usando la descomposición LU.
    :param matrix: Matriz de coeficientes (lista de listas).
    :param results: Vector de resultados (lista).
    :return: Solución del sistema (lista).
    """
    P, L, U = lu(matrix)
    # Resolver Ly = Pb
    y = np.linalg.solve(L, np.dot(P, results))
    # Resolver Ux = y
    x = np.linalg.solve(U, y)
    return x


import unittest
from solver.nonlinear.jacobi import jacobi_method
from solver.nonlinear.gauss_seidel import gauss_seidel_method
from solver.nonlinear.bisection import bisection_method

class TestNonlinearMethods(unittest.TestCase):

    def test_jacobi_method(self):
        equations = [
            "x + y = 10",
            "2*x + 3*y = 24"
        ]
        initial_guess = [0, 0]
        solution = jacobi_method(equations, initial_guess)
        self.assertAlmostEqual(solution[0], 6.0, places=1)
        self.assertAlmostEqual(solution[1], 4.0, places=1)

    def test_gauss_seidel_method(self):
        equations = [
            "x + y = 10",
            "2*x + 3*y = 24"
        ]
        initial_guess = [0, 0]
        solution = gauss_seidel_method(equations, initial_guess)
        self.assertAlmostEqual(solution[0], 6.0, places=1)
        self.assertAlmostEqual(solution[1], 4.0, places=1)

    def test_bisection_method(self):
        func = lambda x: x**2 - 4
        a, b = 0, 3
        root = bisection_method(func, a, b)
        self.assertAlmostEqual(root, 2.0, places=1)

if __name__ == '__main__':
    unittest.main()


import numpy as np

def gauss_seidel(matrix, results, initial_guess, tolerance=1e-10, max_iterations=100):
    """
    Resuelve un sistema de ecuaciones lineales usando el método de Gauss-Seidel.
    :param matrix: Matriz de coeficientes (lista de listas).
    :param results: Vector de resultados (lista).
    :param initial_guess: Vector inicial (lista).
    :param tolerance: Tolerancia para la convergencia.
    :param max_iterations: Número máximo de iteraciones.
    :return: Solución del sistema (lista).
    """
    n = len(matrix)
    x = initial_guess[:]
    for _ in range(max_iterations):
        x_new = x[:]
        for i in range(n):
            sum_ = sum(matrix[i][j] * x_new[j] for j in range(n) if j != i)
            x_new[i] = (results[i] - sum_) / matrix[i][i]
        if np.linalg.norm(np.array(x_new) - np.array(x), ord=np.inf) < tolerance:
            return x_new
        x = x_new
    raise ValueError("El método de Gauss-Seidel no converge.")


# Importar los métodos desde los módulos correspondientes
from .nonlinear.bisection import bisection_method
from .linear.gauss_jordan import gauss_jordan
from .linear.gauss_elimination import gauss_elimination
from .linear.cramer_rule import cramer_rule
from .linear.lu_decomposition import lu_decomposition
from .nonlinear.jacobi import jacobi_method
from .nonlinear.gauss_seidel import gauss_seidel


import unittest
from solver.linear.gauss_elimination import gauss_elimination
from solver.linear.gauss_jordan import gauss_jordan
from solver.linear.cramer_rule import cramer_rule
from solver.linear.lu_decomposition import lu_decomposition

class TestLinearMethods(unittest.TestCase):

    def test_gauss_elimination(self):
        matrix = [[2, 1, -1], [3, 2, -1], [1, 1, 1]]
        b = [8, 13, 3]
        result = gauss_elimination(matrix, b)
        expected = [2, 3, 1]
        self.assertEqual(result, expected)

    def test_gauss_jordan(self):
        matrix = [[2, 1, -1], [3, 2, -1], [1, 1, 1]]
        b = [8, 13, 3]
        result = gauss_jordan(matrix, b)
        expected = [2, 3, 1]
        self.assertEqual(result, expected)

    def test_cramer_rule(self):
        matrix = [[2, 1, -1], [3, 2, -1], [1, 1, 1]]
        b = [8, 13, 3]
        result = cramer_rule(matrix, b)
        expected = [2, 3, 1]
        self.assertEqual(result, expected)

    def test_lu_decomposition(self):
        matrix = [[2, 1, -1], [3, 2, -1], [1, 1, 1]]
        b = [8, 13, 3]
        result = lu_decomposition(matrix, b)
        expected = [2, 3, 1]
        self.assertEqual(result, expected)

if __name__ == '__main__':
    unittest.main()


def gauss_elimination(matrix, results):
    """
    Resuelve un sistema de ecuaciones lineales usando el método de eliminación de Gauss.
    :param matrix: Matriz de coeficientes (lista de listas).
    :param results: Vector de resultados (lista).
    :return: Solución del sistema (lista).
    """
    n = len(matrix)
    for i in range(n):
        # Pivoteo
        for j in range(i + 1, n):
            factor = matrix[j][i] / matrix[i][i]
            for k in range(i, n):
                matrix[j][k] -= factor * matrix[i][k]
            results[j] -= factor * results[i]

    # Sustitución hacia atrás
    x = [0] * n
    for i in range(n - 1, -1, -1):
        x[i] = results[i]
        for j in range(i + 1, n):
            x[i] -= matrix[i][j] * x[j]
        x[i] /= matrix[i][i]
    return x


def bisection_method(func, a, b, tolerance=1e-10, max_iterations=100):
    """
    Encuentra una raíz de una función no lineal usando el método de bisección.
    :param func: Función a evaluar.
    :param a: Límite inferior del intervalo.
    :param b: Límite superior del intervalo.
    :param tolerance: Tolerancia para la convergencia.
    :param max_iterations: Número máximo de iteraciones.
    :return: Raíz de la función.
    """
    if func(a) * func(b) >= 0:
        raise ValueError("El intervalo no contiene una raíz.")

    for _ in range(max_iterations):
        c = (a + b) / 2
        if abs(func(c)) < tolerance or (b - a) / 2 < tolerance:
            return c
        if func(a) * func(c) < 0:
            b = c
        else:
            a = c
    raise ValueError("El método de bisección no converge.")



import numpy as np

def cramer_rule(matrix, results):
    """
    Resuelve un sistema de ecuaciones lineales usando la regla de Cramer.
    :param matrix: Matriz de coeficientes (lista de listas).
    :param results: Vector de resultados (lista).
    :return: Solución del sistema (lista).
    """
    det_main = np.linalg.det(matrix)
    if det_main == 0:
        raise ValueError("El sistema no tiene solución única (determinante = 0).")

    n = len(matrix)
    solutions = []
    for i in range(n):
        temp_matrix = [row[:] for row in matrix]
        for j in range(n):
            temp_matrix[j][i] = results[j]
        solutions.append(np.linalg.det(temp_matrix) / det_main)
    return solutions


import pathlib
from setuptools import setup, find_packages 
HERE = pathlib.Path(__file__).parent
LONG_DESCRIPTION = (HERE / "README.md").read_text(encoding='utf-8') 
LONG_DESC_TYPE = "text/markdown"
setup(
    name='HG22017UNO',
    version='0.2.2',
    author='Jose HErnandez',
    author_email='hg22017@ues.edu.sv',
    description='A library for solving linear and nonlinear systems of equations using various methods.',
    packages=find_packages(),
    install_requires=['numpy>=1.10'],
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)


def gauss_jordan(matrix, results):
    """
    Resuelve un sistema de ecuaciones lineales usando el método de Gauss-Jordan.
    :param matrix: Matriz de coeficientes (lista de listas).
    :param results: Vector de resultados (lista).
    :return: Solución del sistema (lista).
    """
    n = len(matrix)
    for i in range(n):
        # Normalizar la fila actual
        factor = matrix[i][i]
        for j in range(n):
            matrix[i][j] /= factor
        results[i] /= factor

        # Hacer ceros en la columna actual
        for k in range(n):
            if k != i:
                factor = matrix[k][i]
                for j in range(n):
                    matrix[k][j] -= factor * matrix[i][j]
                results[k] -= factor * results[i]
    return results